// src/macro/macro.ts
import {
  isAsyncFunction,
  isFunction,
  isGeneratorFunction,
  VERSION
} from "@typed-macro/shared";
function macro(meta, handler) {
  return {
    name: meta.name,
    handler: normalizeHandler(handler),
    types: renderMacroTypes(meta),
    version: VERSION,
    __is_macro: true
  };
}
function isMacro(o) {
  var _a;
  return (_a = o == null ? void 0 : o.__is_macro) != null ? _a : false;
}
function normalizeHandler(handler) {
  if (!isFunction(handler))
    throw new Error("macro handler should be a function");
  if (isAsyncFunction(handler))
    throw new Error("macro handler should not be an async function");
  if (handler.length === 0)
    throw new Error("macro handler should have at least one parameter");
  return isGeneratorFunction(handler) ? function* (call) {
    yield* handler(call.ctx, call.babel, call.helper);
  } : function* (call) {
    yield call.ctx.args;
    handler(call.ctx, call.babel, call.helper);
  };
}
function renderMacroTypes({ name, types, signatures }) {
  return [
    types.join("\n"),
    signatures.map((s) => s.comment ? `/** ${s.comment} */
export function ${name}${s.signature}` : `export function ${name}${s.signature}`).join("\n")
  ].filter((t) => !!t).join("\n");
}

// src/macro/builder.ts
import { validateFnName } from "@typed-macro/shared";
function defineMacro(name) {
  if (!validateFnName(name))
    throw new Error(`'${name}' is not a valid macro name!`);
  const meta = {
    name,
    signatures: [],
    types: []
  };
  const builder = {
    withCustomType(typeDefinition) {
      meta.types.push(typeDefinition);
      return builder;
    },
    withSignature(signature, comment) {
      if (!signature)
        throw new Error(`Signature for macro '${name}' can not be an empty string`);
      meta.signatures.push({ signature, comment });
      return builder;
    },
    withHandler(handler) {
      if (meta.signatures.length === 0)
        throw new Error(`Please call .withSignature() before .withHandler() to specify at least one signature for macro '${name}'`);
      return macro(meta, handler);
    }
  };
  return builder;
}

// src/provider/provider.ts
import { VERSION as VERSION2 } from "@typed-macro/shared";
function isMacroProvider(o) {
  var _a;
  return (_a = o == null ? void 0 : o.__is_macro_provider) != null ? _a : false;
}
function macroProvider(provider) {
  provider.__is_macro_provider = true;
  provider.version = VERSION2;
  return provider;
}

// src/provider/builder.ts
function defineMacroProvider(raw) {
  return macroProvider(raw);
}
export {
  defineMacro,
  defineMacroProvider,
  isMacro,
  isMacroProvider
};
